function lerp_color(colors, position)
    local colorCount   = #colors
    
    if colorCount == 1 then
        return colors[1]
    end
    
    position = math.max(0, math.min(position, 1))
    
    local chooseColors = (colorCount - 1) * position
    
    if chooseColors == math.round(chooseColors) then
        return colors[math.round(chooseColors)+1]
    end
    
    local leftColor  = colors[math.floor(chooseColors)+1]
    local rightColor = colors[math.ceil(chooseColors)+1]
    local between    = chooseColors - math.floor(chooseColors)
    
    if not leftColor and not rightColor then
        return Color(0, 0, 0)
    end
    
    if not leftColor then return rightColor end
    if not rightColor then return leftColor end
    
    local redToRight   = rightColor.r - leftColor.r
    local greenToRight = rightColor.g - leftColor.g
    local blueToRight  = rightColor.b - leftColor.b
    
    return Color(leftColor.r + (between * redToRight), leftColor.g + (between * greenToRight), leftColor.b + (between * blueToRight))
end


-- Made to be functionally identical to Python's slicing
function table.pyslice(t, left, right, step)
    ret    = {}
    length = #t
    
    if type(step) == "nil" then   step = 1   end
    
    if step < 0 then
        if type(left)  == "nil" then   left  = length   end
        if type(right) == "nil" then   right = 0        end
    
    else
        if type(left)  == "nil" then   left  = 1            end
        if type(right) == "nil" then   right = 1 + length   end
    end
    
    if left  < 1 then   left  = left  + length + 1   end
    if right < 0 then   right = right + length + 1   end
    
    left  = math.max(1, math.min(left,  length))
    right = math.max(0, math.min(right, length + 1))
    
    for i = left, right, step do
        if i == right then break end
        table.insert(ret, t[i])
    end
    
    return ret
end



local brouzoufTextProps  = {
    font = tweak_data.menu.medium_font,
    font_size = 22 * tweak_data.scale.small_font_multiplier,
    layer = -10,
    align = "left",
    vertical = "top",
    x = 0,
    y = 0,
}

local centerTextProps = {
    font = tweak_data.menu.pd2_massive_font,
    font_size = 56,
    layer = -10,
    align = "center",
    vertical = "center",
    text = "",
}

local MAXLINESTOKEEP = 4
local TYPESPEED      = 40
local COLORFADETIME  = 1
local FADECOLORS     = {Color.white,     Color("ffea80")}
local CENTERCOLORS   = {Color("ffe14d"), Color("ffefbf")}
local CENTERLINES    = 3

local SPECIALKILLS = 
{
    "You've killed a Sergeant",
    "You've killed a Captain",
    "You've killed a Colonel",
    "You've killed a General",
    "You've killed a Master",
}

local BEASTKILLS = 
{
    "You've killed a Berserker",
    "You've killed a Dominator",
    "You've killed a Pack Chief",
    "You've killed a Horde Chief",
    "You've killed a Supreme Beast",
}

local RAREKILLS = 
{
    "You've killed a cataphile",
    "You've killed a necrophile",
    "You've killed an adventurer",
    "You've killed a junkie",
    "You've killed a strange man",
    "You've killed a crook",
    "You've killed a corrupt",
    "You've killed a traitor",
    "You've killed a infiltrator",
    "You've killed a masked demon",
    "You've killed a homunculus",
    "You've killed a bastard",
    "You've killed a Holy man",
    "You've killed a lama",
    "You've killed a geek",
    "You've killed a snake-man",
    "You've killed an alcoholic",
    "You've killed a sick person",
    "You've killed a wreck",
    "You've killed a dude who looked like you",
    "You've killed a necromancer",
    "You've killed a good and caring daddy",
    "You've killed a video games censor",
    "You've killed a soccer player",
    "You've killed a ninja",
    "You've killed an android",
    "You've killed a mutant",
    "You've killed a irradiated dude",
    "You've killed a contaminated guy",
    "You've killed a dead man!",
    "You've killed a little-Grey"
}

function HUDManager:myLegsAreOK()
    self.brouzoufLines = {}
    self.brouzoufPanel = self._saferect:panel()
    self.brouzoufText  = self.brouzoufPanel:text(brouzoufTextProps)
    
    self.centerWorkspace = Overlay:newgui():create_screen_workspace()
    self.centerPanel     = self.centerWorkspace:panel()
    
    self.centerLines = {}
    
    for i = 1, CENTERLINES do
        table.insert(self.centerLines,
        {
            textObj   = self.centerPanel:text(centerTextProps),
            line      = "",
            addedTime = 0,
            inUse     = false,
        })
    end
end


function HUDManager:addCenterText(message)
    local t = TimerManager:game():time()
    
    for _, data in pairs(self.centerLines) do
        if not data.inUse then
            data.line      = message
            data.addedTime = t
            data.inUse     = true
            return
        end
    end
    
    -- didn't find one, so we're gonna make room for one
    
    for _, data in pairs(self.centerLines) do
        data.inUse = false
    end
    
    self:addCenterText(message)
end



function HUDManager:youGainBrouzouf(message)
    local t = TimerManager:game():time()
    message = message or "You gain Brouzouf"
    table.insert(self.brouzoufLines, {message = message, t = t})
end

function HUDManager:youKilledABeast()
    local randMessage = BEASTKILLS[math.random(1, #BEASTKILLS)]
    self:addCenterText(randMessage)
end

function HUDManager:youKilledASpecial()
    local randMessage = SPECIALKILLS[math.random(1, #SPECIALKILLS)]
    self:addCenterText(randMessage)
end

function HUDManager:youKilledAGoodAndCaringDaddy()
    local rand = math.random(1, 400)
    
    if rand == 1 then
        local randMessage = RAREKILLS[math.random(1, #RAREKILLS)]
        self:addCenterText(randMessage)
    end
end
    


function HUDManager:didIGainBrouzouf(t, dt)
    local fontSize = BrouzoufSettings:get("textsize")
    local ypos     = BrouzoufSettings:get("ypos")
    
    self.brouzoufText:set_font_size(fontSize * tweak_data.scale.small_font_multiplier)
    self.brouzoufText:set_y(self:saferect_h() * (ypos / 100))

    local printLines = table.pyslice(self.brouzoufLines, -MAXLINESTOKEEP)
    local lineCount  = #printLines
    
    local displayText   = ""
    local displayColors = {}
    
    for i, data in ipairs(printLines) do
        local newLine
        local addedTime = data.t
        local timeDiff  = t - addedTime
        
        local charsToShow = math.max(0, math.floor(TYPESPEED * timeDiff))
        newLine = string.sub(data.message, 1, charsToShow)
        
        if newLine ~= data.message then
            newLine = newLine .. "_"
        end
        
        if i > 1 then
            displayText = displayText .. "\n"
        end
        
        local colorToUse = lerp_color(FADECOLORS, timeDiff / COLORFADETIME)
        
        local h,s,v = rgb_to_hsv(colorToUse.r, colorToUse.g, colorToUse.b)
        local satIndex = 0.5 * (i + 4 - math.min(lineCount, MAXLINESTOKEEP))
        s = s * (1 - (satIndex / MAXLINESTOKEEP))
        
        colorToUse = Color(hsv_to_rgb(h,s,v))
        
        local colorStart = string.len(displayText)
        displayText = displayText .. newLine
        local colorEnd   = string.len(displayText)
        
        table.insert(displayColors, {colorToUse, colorStart, colorEnd})
    end
    
    self.brouzoufText:clear_range_color(1, self.brouzoufText:text():len())
    self.brouzoufText:set_text(displayText)
    
    for i, data in ipairs(displayColors) do
        local color, left, right = data[1], data[2], data[3]
        self.brouzoufText:set_range_color(left, right, color)
    end
end


function HUDManager:handleCenterText(t, panel, textData, offset)
    local textObj   = textData.textObj
    local addedTime = textData.addedTime
    local line      = textData.line
    
    local res     = RenderSettings.resolution
    panel:set_size(res.x, res.y)
    panel:set_position(0, 0)
    
    local centerX = res.x * 0.5
    local centerY = res.y * 0.65
    
    local timeDiff  = t - addedTime
    local alpha     = 1
    
    if timeDiff < 0.25 then
        alpha = timeDiff * 4
        
    elseif timeDiff > 3 then
        alpha = 7 - (2 * timeDiff)
        textData.inUse = false
    end
    
    alpha = math.max(0, math.min(1, alpha))
    local newColor = lerp_color(CENTERCOLORS, alpha):with_alpha(alpha)
    
    textObj:set_text(line)
    textObj:set_color(newColor)
    textObj:set_center(centerX, centerY + offset)
    textObj:set_font_size(56 * tweak_data.scale.default_font_multiplier)
end



local oldInit = HUDManager.init
function HUDManager:init(...)
    oldInit(self, ...)
    self:myLegsAreOK()
end


local oldUpdate = HUDManager.update
function HUDManager:update(t, dt)
    oldUpdate(self, t, dt)
    
    self:didIGainBrouzouf(t, dt)
    
    for i, data in pairs(self.centerLines) do
        self:handleCenterText(t, self.centerPanel, data, i * 66)
    end
end