local BSettingsManager = class()

function BSettingsManager:init()
    self.SAVEDIR  = SavePath
    self.SAVEPATH = self.SAVEDIR .. "/brouzouf.txt"
    self.MENUPATH = ModPath .. "/menu.json"
    
    local keys    = {"ypos", "textsize", "docriticals", "doenemykills"}
    self._data    = {}
    self._keys    = {}
    
    for i, k in pairs(keys) do self._keys[k] = true end
    self:loadSettings()
    
    MenuHelper:LoadFromJsonFile(self.MENUPATH, self, self._data)
    Hooks:Add("LocalizationManagerPostInit", "brouzouf_loc", self.addLocalization)
    self:setupCallbacks()
end


function BSettingsManager:get(option)
    if self._keys[option] then
        return self._data[option]
    end
end

function BSettingsManager:set(option, value)
    if self._keys[option] then
        self._data[option] = value
    end
end


function BSettingsManager:loadSettings()
    local file = io.open(self.SAVEPATH)
    if not file then
        self:defaultSettings()
        return
    end
    
    local vals = json.decode(file:read("*all"))
    
    for k, v in pairs(vals) do
        if self._keys[k] then
            self._data[k] = v
        end
    end
    
    file:close()
end

function BSettingsManager:saveSettings()
    if not file.DirectoryExists(self.SAVEDIR) then return end
    
    local file = io.open(self.SAVEPATH, "w")
    if not file then return end
    
    file:write(json.encode(self._data))
    file:close()
end

function BSettingsManager:defaultSettings()
    local menuFile = io.open(self.MENUPATH)
    local menu = json.decode(menuFile:read("*all"))
    menuFile:close()
    
    for i, item in pairs(menu.items) do
        self:set(item.value, item.default_value)
    end
    
    self:saveSettings()
end


function BSettingsManager.addLocalization(loc)
    loc:add_localized_strings({
        brouzoufmenu_title              = "You gain Brouzouf",
        brouzoufmenu_desc               = "Dis gonna be hardcore man, an ultra-violent party.",
        
        brouzoufmenu_ypos_title         = "Corner text height",
        brouzoufmenu_ypos_desc          = "how high is your brouzouf",
        
        brouzoufmenu_textsize_title     = "Corner text size",
        brouzoufmenu_textsize_desc      = "how big is your brouzouf",
        
        brouzoufmenu_docriticals_title  = "Critical messages",
        brouzoufmenu_docriticals_desc   = "Bullshit! Ultra-failed attack!",
        
        brouzoufmenu_doenemykills_title = "Special enemy titles on kill",
        brouzoufmenu_doenemykills_desc  = "You've killed a good and caring daddy",
    })
end


function BSettingsManager:setupCallbacks()
    local function setupHandler(key)
        return function(menuSelf, item)
            if item._type == "toggle" then
                self:set(key, item:value() == "on" and true or false)
            else
                self:set(key, item:value())
            end
            
            self:saveSettings()
        end
    end
    
    for k, _ in pairs(self._keys) do
        MenuCallbackHandler["brouzouf_" .. k] = setupHandler(k)
    end
end



BrouzoufSettings = BSettingsManager:new()